"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDiscriminatingMutexConcurrencyController = void 0;
const async_mutex_1 = require("async-mutex");
const mutexMap = new Map();
/**
 * Use multiple mutex controllers - one per discriminator
 * @param discriminator The discriminator to find or create a mutex for
 * @returns The controller to acquire and release the mutex for a specific discriminator
 */
const createDiscriminatingMutexConcurrencyController = (discriminator) => {
    return {
        /** Acquire a lock (if any) and return a function to release it. */
        acquire: (message) => {
            const d = discriminator(message);
            let mutex = mutexMap.get(d);
            if (mutex) {
                return mutex.acquire();
            }
            mutex = new async_mutex_1.Mutex();
            mutexMap.set(d, mutex);
            return mutex.acquire();
        },
        /** Cancel all pending locks. */
        cancel: () => {
            for (const mutex of mutexMap.values()) {
                mutex.cancel();
            }
            mutexMap.clear();
        },
    };
};
exports.createDiscriminatingMutexConcurrencyController = createDiscriminatingMutexConcurrencyController;
//# sourceMappingURL=create-discriminating-mutex-concurrency-controller.js.map