"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeOutboxListener = void 0;
const logical_replication_listener_1 = require("../replication/logical-replication-listener");
const concurrency_strategy_1 = require("../strategies/concurrency-strategy");
const message_processing_timeout_strategy_1 = require("../strategies/message-processing-timeout-strategy");
/**
 * Initialize the listener to watch for outbox table inserts via logical replication.
 * @param config The configuration object with required values to connect to the WAL.
 * @param sendMessage This function is called in which you should actually send the message through a message bus or other means.
 * @param logger A logger instance for logging trace up to error logs
 * @param strategies Strategies to provide custom logic for handling specific scenarios
 * @returns Functions for a clean shutdown and to help testing "outages" of the outbox listener
 */
const initializeOutboxListener = (config, sendMessage, logger, strategies) => {
    const logErrors = (_m, error) => __awaiter(void 0, void 0, void 0, function* () {
        logger.error(error, 'An error ocurred while handling an outbox message.');
        return 'transient_error';
    });
    return (0, logical_replication_listener_1.createLogicalReplicationListener)(config, sendMessage, logErrors, logger, applyDefaultStrategies(strategies, config), 'outbox');
};
exports.initializeOutboxListener = initializeOutboxListener;
const applyDefaultStrategies = (strategies, config) => {
    var _a, _b;
    return ({
        concurrencyStrategy: (_a = strategies === null || strategies === void 0 ? void 0 : strategies.concurrencyStrategy) !== null && _a !== void 0 ? _a : (0, concurrency_strategy_1.defaultConcurrencyStrategy)(),
        messageProcessingTimeoutStrategy: (_b = strategies === null || strategies === void 0 ? void 0 : strategies.messageProcessingTimeoutStrategy) !== null && _b !== void 0 ? _b : (0, message_processing_timeout_strategy_1.defaultMessageProcessingTimeoutStrategy)(config),
    });
};
//# sourceMappingURL=outbox-listener.js.map